#include <stdarg.h>
#include "main.h"
#include "LiquidCrystal.h"

#define STM32_TICKS_PER_US	    192
#define STM32_DELAY_US_MULT	    (STM32_TICKS_PER_US/3)

#define	 delay_ms(x) HAL_Delay(x)

static void   LiquidCrystal_command( unsigned char );
static void   LiquidCrystal_write( unsigned char );
static void   LiquidCrystal_sendInitCommand( unsigned char );
static void   LiquidCrystal_write4bits( unsigned char );
static void   LiquidCrystal_pulseEnable( int );

void
delay_us( unsigned int usec )
{
  uint32_t      us = SystemCoreClock / 3000000 * usec;

  /* fudge for function call overhead */
  // us--;
  asm volatile  ( "   mov r0, %[us]	    \n\t"
      "1: subs r0, #1	    \n\t"
      "	  bhi 1b		    \n\t"::[us] "r"( us ):"r0" );
}

// When the display powers up, it is configured as follows:
//
// 1. Display clear
// 2. Function set:
//    DL = 1; 8-bit interface data
//    N = 0; 1-line display
//    F = 0; 5x8 dot character font
// 3. Display on/off control:
//    D = 0; Display off
//    C = 0; Cursor off
//    B = 0; Blinking off
// 4. Entry mode set:
//    I/D = 1; Increment by 1
//    S = 0; No shift
//

unsigned char _displayfunction;
unsigned char _displaycontrol;
unsigned char _displaymode;

unsigned char _initialized;

unsigned char _numlines, _currline;
int           delayAfterCommand = 50;

void
LiquidCrystal_begin( unsigned char cols, unsigned char lines )
{
  _numlines = lines;
  _currline = 0;

  // SEE PAGE 45/46 FOR INITIALIZATION SPECIFICATION! according to
  // datasheet, we need at least 40ms after power rises above 2.7V
  // before sending commands. Arduino can turn on way before 4.5V so
  // we'll wait 50
  delay_ms( 50 );

  // Init 4 bits communication

  // Now we pull both RS and R/W low to begin commands
  pinReset( LCD_RS_GPIO_Port, LCD_RS_Pin );
  pinReset( LCD_E_GPIO_Port, LCD_E_Pin );

  // this is according to the hitachi HD44780 datasheet
  // page 45 figure 23

  // Send function set command sequence
  LiquidCrystal_sendInitCommand( LCD_FUNCTIONSET | LCD_8BITMODE );
  delay_ms( 5 );		// wait more than 4.1ms

  // second try
  LiquidCrystal_sendInitCommand( LCD_FUNCTIONSET | LCD_8BITMODE );
  delay_ms( 1 );

  // third go
  LiquidCrystal_sendInitCommand( LCD_FUNCTIONSET | LCD_8BITMODE );
  delay_ms( 1 );

  // And set the 4 bits mode
  LiquidCrystal_sendInitCommand( LCD_FUNCTIONSET | LCD_4BITMODE );
  delay_ms( 1 );

  if( lines > 1 )
  {
    _displayfunction = LCD_4BITMODE | LCD_2LINE | LCD_5x8DOTS;
  }
  else
  {
    _displayfunction = LCD_4BITMODE | LCD_1LINE | LCD_5x8DOTS;
  }

  // finally, set # lines, font size, etc.
  LiquidCrystal_command( LCD_FUNCTIONSET | _displayfunction );

  LiquidCrystal_command( LCD_DISPLAYCONTROL );

  // clear
  LiquidCrystal_command( LCD_CLEARDISPLAY );	// clear display, set cursor position to zero
  delay_ms( 2 );

  // Initialize to default text direction (for romance languages)
  _displaymode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;
  // set the entry mode
  LiquidCrystal_command( LCD_ENTRYMODESET | _displaymode );

  // turn the display on with no cursor or blinking default
  _displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;
  LiquidCrystal_display(  );

#ifndef BOOTLOADER
  delayAfterCommand = 38;
#endif

}

/********** high level commands, for the user! */
void
LiquidCrystal_clear(  )
{

  LiquidCrystal_command( LCD_CLEARDISPLAY );	// clear display, set cursor position to zero
  delay_ms( 2 );		// this command takes a long time!
}

void
LiquidCrystal_home( void )
{
  LiquidCrystal_command( LCD_RETURNHOME );	// set cursor position to zero
  delay_ms( 2 );		// this command takes a long time!
}

void
LiquidCrystal_setCursor( unsigned char col, unsigned char row )
{
  int           row_offsets[] = { 0x00, 0x40, 0x14, 0x54 };
  if( row > _numlines )
  {
    row = _numlines - 1;	// we count rows starting w/0
  }
  LiquidCrystal_command( LCD_SETDDRAMADDR | ( col + row_offsets[row] ) );
}

// Turn the display on/off (quickly)
void
LiquidCrystal_noDisplay( void )
{
  _displaycontrol &= ~LCD_DISPLAYON;
  LiquidCrystal_command( LCD_DISPLAYCONTROL | _displaycontrol );
}

void
LiquidCrystal_display( void )
{
  _displaycontrol |= LCD_DISPLAYON;
  LiquidCrystal_command( LCD_DISPLAYCONTROL | _displaycontrol );
}

// Turns the underline cursor on/off
void
LiquidCrystal_noCursor( void )
{
  _displaycontrol &= ~LCD_CURSORON;
  LiquidCrystal_command( LCD_DISPLAYCONTROL | _displaycontrol );
}

void
LiquidCrystal_cursor( void )
{
  _displaycontrol |= LCD_CURSORON;
  LiquidCrystal_command( LCD_DISPLAYCONTROL | _displaycontrol );
}

// Turn on and off the blinking cursor
void
LiquidCrystal_noBlink( void )
{
  _displaycontrol &= ~LCD_BLINKON;
  LiquidCrystal_command( LCD_DISPLAYCONTROL | _displaycontrol );
}

void
LiquidCrystal_blink( void )
{
  _displaycontrol |= LCD_BLINKON;
  LiquidCrystal_command( LCD_DISPLAYCONTROL | _displaycontrol );
}

// These commands scroll the display without changing the RAM
void
LiquidCrystal_scrollDisplayLeft( void )
{
  LiquidCrystal_command( LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT );
}

void
LiquidCrystal_scrollDisplayRight( void )
{
  LiquidCrystal_command( LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT );
}

// This is for text that flows Left to Right
void
LiquidCrystal_leftToRight( void )
{
  _displaymode |= LCD_ENTRYLEFT;
  LiquidCrystal_command( LCD_ENTRYMODESET | _displaymode );
}

// This is for text that flows Right to Left
void
LiquidCrystal_rightToLeft( void )
{
  _displaymode &= ~LCD_ENTRYLEFT;
  LiquidCrystal_command( LCD_ENTRYMODESET | _displaymode );
}

// This will 'right justify' text from the cursor
void
LiquidCrystal_autoscroll( void )
{
  _displaymode |= LCD_ENTRYSHIFTINCREMENT;
  LiquidCrystal_command( LCD_ENTRYMODESET | _displaymode );
}

// This will 'left justify' text from the cursor
void
LiquidCrystal_noAutoscroll( void )
{
  _displaymode &= ~LCD_ENTRYSHIFTINCREMENT;
  LiquidCrystal_command( LCD_ENTRYMODESET | _displaymode );
}

// Allows us to fill the first 8 CGRAM locations
// with custom characters
void
LiquidCrystal_createChar( unsigned char location, unsigned char charmap[] )
{
  int i;
  location &= 0x7;		// we only have 8 locations 0-7
  for( i = 0; i < 8; i++ )
  {
    LiquidCrystal_command( LCD_SETCGRAMADDR | ( location << 3 ) | i );
    LiquidCrystal_write( charmap[i] );
  }
}

void
LCD_char( unsigned char col, unsigned char row, unsigned char ch )
{
  LiquidCrystal_setCursor(col, row );
  LiquidCrystal_write( ch );
}

int
LCD_print( unsigned char col, unsigned char row, char *fmt, ... )
{
  char          line[64];
  int           ret;
  char         *s = line;
  va_list       ap;

  LiquidCrystal_setCursor(col, row );
  va_start( ap, fmt );
  ret = vsnprintf( line, 64, fmt, ap );
  va_end( ap );
  while( *s )
    LiquidCrystal_write( *s++ );
  return ret;
}

/************ low level data pushing commands **********/

static void
LiquidCrystal_sendInitCommand( unsigned char value )
{
  pinReset( LCD_RS_GPIO_Port, LCD_RS_Pin );

  LiquidCrystal_write4bits( value >> 4 );
  LiquidCrystal_pulseEnable( delayAfterCommand );
}

// write command with automatic 4/8-bit selection
static void
LiquidCrystal_command( unsigned char value )
{
  pinReset( LCD_RS_GPIO_Port, LCD_RS_Pin );
  LiquidCrystal_write4bits( value >> 4 );
  LiquidCrystal_pulseEnable( 1 );
  LiquidCrystal_write4bits( value );
  LiquidCrystal_pulseEnable( delayAfterCommand );
}

// write data with automatic 4/8-bit selection
static void
LiquidCrystal_write( unsigned char value )
{
  pinSet( LCD_RS_GPIO_Port, LCD_RS_Pin );
  LiquidCrystal_write4bits( value >> 4 );
  LiquidCrystal_pulseEnable( 1 );
  LiquidCrystal_write4bits( value );
  LiquidCrystal_pulseEnable( delayAfterCommand );
}

static void
LiquidCrystal_pulseEnable( int delay )
{
  // LCD_E_Pin should already be LOW (unless someone else messed
  // with it), so don't sit around waiting for long.
  pinReset( LCD_E_GPIO_Port, LCD_E_Pin );
  delay_us( 1 );

  // Enable pulse must be > 450 ns.  Value chosen here according to
  // the following threads:
  // http://forums.leaflabs.com/topic.php?id=640
  // http://forums.leaflabs.com/topic.php?id=512
  pinSet( LCD_E_GPIO_Port, LCD_E_Pin );
  delay_us( 1 );
  pinReset( LCD_E_GPIO_Port, LCD_E_Pin );
  // Commands needs > 37us to settle.
  delay_us( delay );
}

static void
LiquidCrystal_write4bits( unsigned char value )
{
  u16           port = LCD_DB7_GPIO_Port->ODR & ~0xf;
  LCD_DB7_GPIO_Port->ODR = port | ( value & 0xf );	// LCD_DB4_Pin is pin0
}
